package com.hisschemoller.audio.model 
{
	import com.hisschemoller.audio.model.parameter.MappingNumberFaderTaper;
	import com.hisschemoller.audio.model.parameter.Parameter;
	import com.hisschemoller.audio.notification.AudioNotifications;
	import com.hisschemoller.audio.utils.Conversions;

	import org.puremvc.as3.multicore.interfaces.IProxy;
	import org.puremvc.as3.multicore.patterns.proxy.Proxy;

	import flash.utils.getQualifiedClassName;

	/**
	 * @author Wouter Hisschemoller
	 * Feb 19, 2010
	 * 
	 * The complete output signal can be attenuated or amplified here.
	 * Uses input in decibel.
	 */
	public class AudioOutputProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = getQualifiedClassName(AudioOutputProxy);
		private var _decibel : Parameter = new Parameter(new MappingNumberFaderTaper(-200, 10), -0.3);
		private var _gain : Number;
		
		public function AudioOutputProxy()
		{
			super(NAME);
		}
		
		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_gain = Conversions.decibelToAmplitude(_decibel.value);
		}
		
		/**
		 * Multiply signal by _gain.
		 */
		public function process(buffer : Vector.<Vector.<Number>>, bufferStartIndex : uint, bufferLength : uint) : void
		{
			var l : Vector.<Number> = buffer[0]; /** left channel */
			var r : Vector.<Number> = buffer[1]; /** right channel */
			var n : int = bufferStartIndex + bufferLength;
			for(var i : int = bufferStartIndex;i < n; ++i)
			{
				l[i] *= _gain;
				r[i] *= _gain;
			}
		}
		
		/**
		 * @return Number: Level in decibel (-200 to 10).
		 */
		public function get decibel() : Number
		{
			return _decibel.value;
		}
		
		/**
		 * @param value: A value in decibel from -200 to 10.
		 */
		public function set outputLevel(value : Number) : void
		{
			_decibel.value = value;
			
			decibelChanged();
		}
		
		/**
		 * @return Number: Level normalized (0 to 1);
		 */
		public function get decibelNormalized() : Number
		{
			return _decibel.valueNormalized;
		}
		
		/**
		 * @param value: A normalized value between 0 and 1
		 */
		public function set decibelNormalized(valueNormalized : Number) : void
		{
			_decibel.valueNormalized = valueNormalized;
			
			decibelChanged();
		}
		
		/**
		 * Update gain (signal multiplier) based on decibel value.
		 */
		private function decibelChanged() : void
		{
			_gain = Conversions.decibelToAmplitude(_decibel.value);
			
			facade.sendNotification(AudioNotifications.AUDIO_OUTPUT_GAIN_CHANGED, _decibel.valueNormalized);
		}
	}
}
