package com.hisschemoller.audio.model.parameter 
{
	import de.popforge.parameter.IMapping;

	/**
	 * @author Wouter Hisschemoller
	 * (c) Feb 21, 2010
	 */
	public class Parameter 
	{
		private var _value : *;
		private var _mapping : IMapping;
		private var _defaultValue : *;

		/**
		 * Creates a Parameter instance.
		 * @param mapping The mapping used to map/mapInverse the normalized value
		 * @param value The default values
		 */
		public function Parameter(mapping : IMapping = null, value : * = null)
		{
			_value = _defaultValue = value;
			_mapping = mapping;
		}

		/**
		 * Sets the current value of the parameter.
		 */
		public function set value(value : *) : void
		{
			_value = value;
		}

		/**
		 * Returns the current value of the parameter
		 */
		public function get value() : *
		{
			return _value;
		}

		/**
		 * Sets the current value of the parameter
		 * by passing a normalized value between 0 and 1
		 * if changed, inform all callbacks
		 * @param normalizedValue A normalized value between 0 and 1
		 */
		public function set valueNormalized(normalizedValue : Number) : void
		{
			_value = _mapping.map(normalizedValue);
		}

		/**
		 * Returns the current normalized value of the parameter
		 * between 0 and 1
		 */
		public function get valueNormalized() : Number
		{
			return _mapping.mapInverse(_value);
		}

		/**
		 * Reset value to its initial default value.
		 */
		public function reset() : void
		{
			value = _defaultValue;
		}
	}
}
