package com.hisschemoller.audio.utils 
{
	import de.popforge.audio.output.Audio;

	import flash.utils.ByteArray;
	/**
	 * @author Wouter Hisschemoller
	 * Jan 16, 2010
	 */
	public class Conversions 
	{
		/**
		 * Covert semitone shift to playback speed multiplier.
		 * For example: -12 returns 0.5, 12 returns 2, 0 returns 1.
		 */
		public static function semitoneToRate(semitone : Number) : Number
		{
			return Math.pow(2, semitone / 12);
		}
		
		/**
         * From: http://www.sengpielaudio.com/calculator-db-volt.htm
         * @param value: amplitude.
         * @return: decibel.
         */
        public static function amplitudeToDecibel(value : Number) : Number
        {
          	// return Math.round(1000000000 * 20 * (Math.log(value / 1) / Math.log(10))) / 1000000000;
          	return 20 * (Math.log(value / 1) / Math.log(10));
        }

        /**
         * From: http://www.sengpielaudio.com/calculator-db-volt.htm
         * @param value: decibel.
         * @return: amplitude.
         */
        public static function decibelToAmplitude(value : Number) : Number
        {
            return Math.exp((value / 20) * Math.log(10));
		}
		
		/**
		 * Convert a byteArray that contains audio samples to a vector of SampleVO objects.
		 * Note that the byteArray is always stereo because that's what the decompilers produce.
		 */
		public static function byteArrayToVector(byteArray : ByteArray, channels : int, bytesPerMonoSample : uint) : Vector.<Vector.<Number>>
		{
			var i : int = -1;
			var numSamples : int = byteArray.length / (bytesPerMonoSample * Audio.STEREO);
			var samples : Vector.<Vector.<Number>> = new Vector.<Vector.<Number>>(2, true);
			samples[0] = new Vector.<Number>(numSamples, true);
			samples[1] = new Vector.<Number>(numSamples, true);
			byteArray.position = 0;
			
			var l : Vector.<Number> = samples[0]; /** left channel */
			var r : Vector.<Number> = samples[1]; /** right channel */
			
			while(++i < numSamples)
			{
				if(channels == Audio.MONO)
				{
					l[i] = byteArray.readFloat();
					r[i] = l[i];
					byteArray.readFloat();
				}
				else if(channels == Audio.STEREO)
				{
					l[i] = byteArray.readFloat();
					r[i] = byteArray.readFloat();
				}
			}
			
			return samples;
		}
		
		/**
		 * @param microSecondsPerQuarterNote: 
		 */
		public static function microSecondsPerQuarterNoteToBPM(microSecondsPerQuarterNote : int) : Number
		{
			return 60 / (microSecondsPerQuarterNote / 1000000);
		}
	}
}
