package com.hisschemoller.sequencer.controller 
{
	import com.hisschemoller.audio.model.AudioDriverProxy;
	import com.hisschemoller.audio.model.AudioEngineProxy;
	import com.hisschemoller.audio.model.AudioLibraryProxy;
	import com.hisschemoller.audio.model.AudioOutputProxy;
	import com.hisschemoller.sequencer.App;
	import com.hisschemoller.sequencer.model.ProjectProxy;
	import com.hisschemoller.sequencer.model.SequencerProxy;
	import com.hisschemoller.sequencer.view.BusyMediator;
	import com.hisschemoller.sequencer.view.ControlsMediator;
	import com.hisschemoller.sequencer.view.EuclidEditorMediator;
	import com.hisschemoller.sequencer.view.EuclidPatternSettingsMediator;
	import com.hisschemoller.sequencer.view.InstrumentSettingsMediator;
	import com.hisschemoller.sequencer.view.SettingsWindowMediator;
	import com.hisschemoller.sequencer.view.components.Busy;
	import com.hisschemoller.sequencer.view.components.Controls;
	import com.hisschemoller.sequencer.view.components.euclidpattern.EuclidEditor;
	import com.hisschemoller.sequencer.view.components.settings.EuclidPatternSettings;
	import com.hisschemoller.sequencer.view.components.settings.InstrumentSettings;
	import com.hisschemoller.sequencer.view.components.settings.SettingsWindow;
	import com.hisschemoller.ui.view.AlertMediator;
	import com.hisschemoller.ui.view.components.Alert;

	import org.puremvc.as3.multicore.interfaces.ICommand;
	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.command.SimpleCommand;

	/**
	 * @author wouter.hisschemoller
	 * Mar 1, 2010
	 */
	public class StartupCommand extends SimpleCommand implements ICommand 
	{
		/**
		 * Register the Proxies and Mediators.
		 */
		override public function execute (note : INotification) : void
		{
			var app : App = note.getBody( ) as App;
			
			/** Create and register proxies. */
			facade.registerProxy( new AudioEngineProxy( ) );
			facade.registerProxy( new AudioOutputProxy( ) );
			facade.registerProxy( new AudioDriverProxy( ) );
			facade.registerProxy( new AudioLibraryProxy( ) );
			facade.registerProxy( new SequencerProxy( ) );
			facade.registerProxy( new ProjectProxy( ) );
			
			/** Create and register mediators. */
			var alert : Alert = new Alert( app );
			facade.registerMediator( new AlertMediator( alert ) );
			
			// var patternEditor : PatternEditor = new PatternEditor( app );
			// facade.registerMediator( new PatternEditorMediator( patternEditor ) );

			var euclidEditor : EuclidEditor = new EuclidEditor( app );
			facade.registerMediator( new EuclidEditorMediator( euclidEditor ) );
			
			var controls : Controls = new Controls( app );
			facade.registerMediator( new ControlsMediator( controls ) );
			
			var settingsWindow : SettingsWindow = new SettingsWindow( app );
			facade.registerMediator( new SettingsWindowMediator( settingsWindow ) );
			
			var trackSettings : EuclidPatternSettings = new EuclidPatternSettings( );
			settingsWindow.addWindow( trackSettings );
			facade.registerMediator( new EuclidPatternSettingsMediator( trackSettings ) );
			
			var instrumentSettings : InstrumentSettings = new InstrumentSettings( );
			settingsWindow.addWindow( instrumentSettings );
			facade.registerMediator( new InstrumentSettingsMediator( instrumentSettings ) );
			
			var busy : Busy = new Busy( app );
			facade.registerMediator( new BusyMediator( busy ) );
			
			/** Load a project. */
			(facade.retrieveProxy( ProjectProxy.NAME ) as ProjectProxy).loadProject( "../xml/project.xml" );
		}
	}
}
