package com.hisschemoller.sequencer.model
{
	import com.hisschemoller.audio.model.AudioEngineProxy;
	import com.hisschemoller.audio.model.AudioLibraryProxy;
	import com.hisschemoller.audio.model.AudioOutputProxy;
	import com.hisschemoller.audio.model.processor.IAudioInstrument;
	import com.hisschemoller.audio.notification.AudioNotifications;
	import com.hisschemoller.audio.utils.delegates.vo.AudioLoaderVO;
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.model.vo.SeqEventVO;
	import com.hisschemoller.sequencer.model.vo.SequenceVO;
	import com.hisschemoller.sequencer.model.vo.TrackVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;
	import com.hisschemoller.ui.model.ICanBeBusy;
	import com.hisschemoller.ui.model.vo.AlertVO;
	import com.hisschemoller.ui.notification.UINotifications;
	import com.hisschemoller.utils.delegates.XMLLoaderDelegate;
	import com.hisschemoller.utils.responder.Responder;
	import com.newgonzo.midi.messages.ChannelMessage;
	import com.newgonzo.midi.messages.MessageStatus;
	import com.newgonzo.midi.messages.VoiceMessage;

	import org.puremvc.as3.multicore.patterns.proxy.Proxy;

	import flash.utils.getQualifiedClassName;
	/**
	 * @author wouter.hisschemoller
	 */
	public class ProjectProxy extends Proxy implements ICanBeBusy
	{
		public static const NAME : String = getQualifiedClassName( ProjectProxy );
		private var _projectXML : XML;
		private var _projectName : String;
		private var _isBusy : Boolean;
		private var _isSetupDone : Boolean;

		public function ProjectProxy (data : Object = null)
		{
			super( NAME, data );
		}

		/**
		 * Load project XML.
		 */
		public function loadProject (projectXMLURL : String) : void
		{
			setBusy( true );
			_isSetupDone = false;

			new XMLLoaderDelegate( new Responder( xmlLoadResult, xmlLoadFault ) ).loadXML( projectXMLURL );
		}

		/**
		 * Called by BusyCommand once all loading is done.
		 */
		public function finishProjectSetup () : void
		{
			if(!_isSetupDone)
			{
				_isSetupDone = true;
				
				/** Send a program change to all instruments. */
				var instrumentsXMLList : XMLList = _projectXML..instrument;
				var audioEngineProxy : AudioEngineProxy = facade.retrieveProxy( AudioEngineProxy.NAME ) as AudioEngineProxy;
				
				/** Loop through all instruments */
				var numInstruments : uint = instrumentsXMLList.length( );
				for(var i : uint = 0; i < numInstruments; i++)
				{
					var instrumentXML : XML = instrumentsXMLList[i];
					var instrument : IAudioInstrument = audioEngineProxy.getInstrumentByID( instrumentXML.@id );
					instrument.programChange( new ChannelMessage( MessageStatus.PROGRAM_CHANGE, 0, int( instrumentXML.@selectedprogram ), 0 ) );
				}
				
				var sequencerProxy : SequencerProxy = facade.retrieveProxy( SequencerProxy.NAME ) as SequencerProxy;
				sequencerProxy.setResolution( _projectXML.@ppqn );
				
				sendNotification( SeqNotifications.SEQ_CHANGE_TEMPO, Number( _projectXML.@tempo ) );
				sendNotification( SeqNotifications.SEQ_SELECT_SEQUENCE, String( _projectXML.sequences.@selected_sequence ) );
				
				var audioOutputProxy : AudioOutputProxy = facade.retrieveProxy( AudioOutputProxy.NAME ) as AudioOutputProxy;
				audioOutputProxy.outputLevel = Number( _projectXML.@outputlevel );
			}
		}

		/**
		 * 
		 */
		public function get isBusy () : Boolean
		{
			return _isBusy;
		}

		/**
		 * 
		 */
		private function setBusy (isBusy : Boolean) : void
		{
			_isBusy = isBusy;

			facade.sendNotification( UINotifications.SET_BUSY, _isBusy );
		}

		/**
		 * XML succesfully loaded.
		 */
		private function xmlLoadResult (xml : XML) : void
		{
			parseProjectXML( xml );
		}

		/**
		 * XML failed to load.
		 */
		private function xmlLoadFault (info : String) : void
		{
			facade.sendNotification( UINotifications.ALERT, new AlertVO( "Project XML load error", info ) );

			setBusy( false );
		}

		/**
		 * 
		 */
		private function parseProjectXML (xml : XML) : void
		{
			/** Set project properties. */
			_projectXML = xml;
			_projectName = xml.@name;
			
			initInstruments( xml..instrument );
			initTracks( xml..track );
			initSequences( xml..sequence );
			initSounds( xml..sound );
			
			setBusy( false );
		}

		/**
		 * Load instruments.
		 */
		private function initInstruments (instrumentsXmlList : XMLList) : void
		{
			/** Loop through all instruments */
			var numInstruments : uint = instrumentsXmlList.length( );
			for( var i : uint = 0; i < numInstruments; i++ )
			{
				// var instrumentXML : XML = instrumentsXmlList[i];

				/** Only PITCHED_SAMPLE type will work in this app. */
				// if( instrumentXML.@type != AudioProcessorType.PITCHED_SAMPLE_PROCESSOR )
				// {
				// 	facade.sendNotification( UINotifications.ALERT, new AlertVO( "Wrong type", "An instrument defined in the settings file is of an unsupported type: " + instrumentXML.@type + ". Please correct the settings file." ) );
				// 	return;
				// }
				sendNotification( AudioNotifications.ADD_INSTRUMENT, instrumentsXmlList[i] as XML );

				/** Create and register the instrument proxy. */
				// facade.registerProxy( new SamplerInstrumentProxy( (instrumentXML.@id), instrumentXML ) );

				/** Add the instrument to the audio engine. */
				// (facade.retrieveProxy( AudioEngineProxy.NAME ) as AudioEngineProxy).addInstrument( SamplerInstrumentProxy( facade.retrieveProxy( instrumentXML.@id ) ) );
			}
		}

		/**
		 * Load sequencer tracks and patterns.
		 */
		private function initTracks (tracksXMLList : XMLList) : void
		{
			var numTracks : int = tracksXMLList.length( );
			for(var i : uint = 0; i < numTracks ; i++)
			{
				var trackVO : TrackVO = new TrackVO( );
				trackVO.id = tracksXMLList[i].@id;
				trackVO.instrumentID = tracksXMLList[i].@instrument_id;
				trackVO.label = tracksXMLList[i].@label;
				
				sendNotification( SeqNotifications.SEQ_ADD_TRACK, trackVO );
			}
		}

		/**
		 * Load sequencer tracks and patterns.
		 */
		private function initSequences (sequencesXMLList : XMLList) : void
		{
			var numSequences : int = sequencesXMLList.length( );
			for(var i : uint = 0; i < numSequences ; i++)
			{
				var sequenceVO : SequenceVO = new SequenceVO( );
				sequenceVO.id = sequencesXMLList[i].@id;
				sequenceVO.length = sequencesXMLList[i].@length;

				var patternsXMLList : XMLList = sequencesXMLList[i]..pattern;
				var numPatterns : int = patternsXMLList.length( );

				for(var j : uint = 0; j < numPatterns ; j++)
				{
					var patternVO : PatternVO = new PatternVO( );
					patternVO.trackID = patternsXMLList[j].@track_id;
					patternVO.length = patternsXMLList[j].@length;
					patternVO.quantization = patternsXMLList[j].@quantization;
					
					var eventsXMLList : XMLList = patternsXMLList[j]..event;
					var numEvents : int = eventsXMLList.length( );
					
					for(var k : uint = 0; k < numEvents ; k++)
					{
						var eventVO : SeqEventVO = new SeqEventVO( );
						eventVO.time = eventsXMLList[k].@time;
						
						switch(String( eventsXMLList[k].@type ))
						{
							case "note_on":
								eventVO.message = new VoiceMessage( MessageStatus.NOTE_ON, 0, int( eventsXMLList[k].@pitch ), int( eventsXMLList[k].@velocity ) );
								break;
								
							case "note_off":
								eventVO.message = new VoiceMessage( MessageStatus.NOTE_OFF, 0, int( eventsXMLList[k].@pitch ), int( eventsXMLList[k].@velocity ) );
								break;
								
							default:
								log( "e Event type not recognized: " + eventsXMLList[k].@type );
								break;
						}

						patternVO.events.push( eventVO );
					}

					sequenceVO.patterns.push( patternVO );
				}
				
				(facade.retrieveProxy( SequencerProxy.NAME ) as SequencerProxy).addSequence( sequenceVO );
			}
		}

		/**
		 * Load audio files.
		 */
		private function initSounds (soundsXMLList : XMLList) : void
		{
			var numSounds : int = soundsXMLList.length( );
			var audioLoaderVector : Vector.<AudioLoaderVO> = new Vector.<AudioLoaderVO>( numSounds, true );

			for(var i : uint = 0; i < numSounds ; i++)
			{
				var audioLoaderVO : AudioLoaderVO = new AudioLoaderVO( soundsXMLList[i].@url );
				audioLoaderVO.id = soundsXMLList[i].@id;

				audioLoaderVector[i] = audioLoaderVO;
			}

			(facade.retrieveProxy( AudioLibraryProxy.NAME ) as AudioLibraryProxy).loadAudioFiles( audioLoaderVector );
		}
	}
}
