package com.hisschemoller.sequencer.model
{
	import com.hisschemoller.audio.model.vo.AudioEventVO;
	import com.hisschemoller.audio.notification.AudioNotifications;
	import com.hisschemoller.sequencer.model.enum.PlaybackType;
	import com.hisschemoller.sequencer.model.vo.PatternEventViewVO;
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.model.vo.PatternViewVO;
	import com.hisschemoller.sequencer.model.vo.PlaybackVO;
	import com.hisschemoller.sequencer.model.vo.QuantizationVO;
	import com.hisschemoller.sequencer.model.vo.SeqEventVO;
	import com.hisschemoller.sequencer.model.vo.SequenceVO;
	import com.hisschemoller.sequencer.model.vo.TempoVO;
	import com.hisschemoller.sequencer.model.vo.TrackVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;

	import org.puremvc.as3.multicore.patterns.proxy.Proxy;

	import flash.utils.getQualifiedClassName;
	/**
	 * @author wouter.hisschemoller
	 * Sep 30, 2010
	 */
	public class SequencerProxy extends Proxy
	{
		public static const NAME : String = getQualifiedClassName( SequencerProxy );
		private var _sequences : Vector.<SequenceVO> = new Vector.<SequenceVO>( );
		private var _selectedSequence : SequenceVO;
		private var _tracks : Vector.<TrackVO> = new Vector.<TrackVO>( );
		private var _quantizations : Vector.<QuantizationVO> = new Vector.<QuantizationVO>( );
		private var _tempoVO : TempoVO = new TempoVO( );
		private var _playbackVO : PlaybackVO = new PlaybackVO( );

		public function SequencerProxy ( data : Object = null )
		{
			super( NAME, data );
		}

		/**
		 * The goal here is to get the events from the tracks that should happen
		 * within the length of the current buffer. And to dispatch these events.
		 * The supported events are NOTE_ON and NOTE_OFF.
		 * @param bufferLength: Length of buffer in samples.
		 */
		public function process ( bufferLength : int ) : void
		{
			tempoVO.bufferLengthInPartsAsNumber = bufferLength / _tempoVO.samplesPerPart;
			var firstPartWithinBuffer : uint;
			var lastPartWithinBuffer : uint;
			_playbackVO.timeAtProcessStart = new Date( ).getTime( );
			_playbackVO.positionInPartsAtProcessStart = _playbackVO.positionInParts;

			switch(_playbackVO.playType)
			{
//				case PlaybackType.PATTERN:
				//					if(_playbackVO.positionInParts + bufferLengthInPartsAsNumber <= _selectedSequence.length)
				//					{
				//						firstPartWithinBuffer = Math.ceil( _playbackVO.positionInParts );
				//						lastPartWithinBuffer = uint( _playbackVO.positionInParts + bufferLengthInPartsAsNumber );
				//						processMessages( firstPartWithinBuffer, lastPartWithinBuffer );
				//						_playbackVO.positionInParts += bufferLengthInPartsAsNumber;
				//					}
				//					else
				//					{
				//						/** The pattern will restart during this buffer. */
				//						var partsBeyondPatternLengthAsNumber : Number = _selectedSequence.length - _playbackVO.positionInParts;
				//
				//						/** Process the last bit of the pattern. */
				//						firstPartWithinBuffer = Math.ceil( _playbackVO.positionInParts );
				//						lastPartWithinBuffer = _selectedSequence.length;
				//						processMessages( firstPartWithinBuffer, lastPartWithinBuffer );
				//						_playbackVO.positionInParts -= _selectedSequence.length;
				//
				//						/** Reset all tracks. */
				//						resetPatterns( );
				//
				//						/** Play remaining first part of the pattern. */
				//						firstPartWithinBuffer = 0;
				//						lastPartWithinBuffer = uint( partsBeyondPatternLengthAsNumber );
				//						processMessages( firstPartWithinBuffer, lastPartWithinBuffer );
				//						_playbackVO.positionInParts += bufferLengthInPartsAsNumber;
				//					}
				//					break;
				case PlaybackType.SONG:
					firstPartWithinBuffer = Math.ceil( _playbackVO.positionInParts );
					lastPartWithinBuffer = uint( _playbackVO.positionInParts + tempoVO.bufferLengthInPartsAsNumber );
					processSequence( firstPartWithinBuffer, lastPartWithinBuffer );
					_playbackVO.positionInParts += tempoVO.bufferLengthInPartsAsNumber;
					break;
			}
		}

		/**
		 * @param value: Resolution in parts per quarter note (PPQN).
		 */
		public function setResolution ( value : int ) : void
		{
			_tempoVO.partsPerQuarterNote = value;
		}

		/**
		 * Add a new track to the sequencer.
		 */
		public function addTrack ( trackVO : TrackVO ) : void
		{
			_tracks.push( trackVO );
		}

		/**
		 * Add a new sequence.
		 */
		public function addSequence ( sequenceVO : SequenceVO ) : void
		{
			_sequences.push( sequenceVO );
		}

		/**
		 * Set all pattern positions to 0 (on the selected sequence).
		 */
		public function resetPatterns () : void
		{
			/** Set all track array indices to zero. */
			var i : int = -1;
			var n : uint = _selectedSequence.patterns.length;
			while(++i < n)
			{
				var patternVO : PatternVO = _selectedSequence.patterns[ i ];
				patternVO.currentEventIndex = 0;
				patternVO.positionInParts = 0;
			}
		}

		/**
		 * 
		 */
		override public function onRegister () : void
		{
			_playbackVO.playType = PlaybackType.SONG;
			
			_quantizations.push( new QuantizationVO( "1/1", 1 ) );
			_quantizations.push( new QuantizationVO( "1/2", 2 ) );
			_quantizations.push( new QuantizationVO( "1/4", 4 ) );
			_quantizations.push( new QuantizationVO( "1/8", 8 ) );
			_quantizations.push( new QuantizationVO( "1/16", 16 ) );
			_quantizations.push( new QuantizationVO( "1/32", 32 ) );
		}

		/**
		 * Set all pattern positions to 0 (on the selected sequence).
		 */
		public function getTrackByID ( trackID : String ) : TrackVO
		{
			var n : uint = _tracks.length;
			while( --n > -1)
			{
				if( _tracks[ n ].id == trackID ) 
				{
					return _tracks[ n ];
				}
			}
			return null;
		}

		/**
		 * Getters and setters.
		 */
		public function get selectedSequence () : SequenceVO
		{
			return _selectedSequence;
		}

		public function set selectedSequence (selectedSequence : SequenceVO) : void
		{
			_selectedSequence = selectedSequence;
		}

		public function get sequences () : Vector.<SequenceVO>
		{
			return _sequences;
		}

		public function get playbackVO () : PlaybackVO
		{
			return _playbackVO;
		}

		public function get tempoVO () : TempoVO
		{
			return _tempoVO;
		}

		public function get tracks () : Vector.<TrackVO>
		{
			return _tracks;
		}

		/**
		 * 
		 */
		private function processSequence ( firstPart : uint, lastPart : uint ) : void
		{
			/** Loop through all patterns. */
			var i : int = -1;
			var numPatterns : uint = _selectedSequence.patterns.length;
			while( ++i < numPatterns )
			{
				var patternVO : PatternVO = _selectedSequence.patterns[ i ];
				var firstPartWithinPattern : uint = Math.ceil( patternVO.positionInParts );
				var lastPartWithinPattern : uint = ( patternVO.positionInParts + tempoVO.bufferLengthInPartsAsNumber ) % patternVO.length;
				
				if( lastPartWithinPattern > firstPartWithinPattern )
				{
					processPattern( patternVO, firstPartWithinPattern, lastPartWithinPattern, patternVO.positionInParts );
				}
				else
				{
					/** If the pattern loops process the last bit, then the first bit of the new loop. */
					processPattern( patternVO, firstPartWithinPattern, patternVO.length, patternVO.positionInParts );
					processPattern( patternVO, 0, lastPartWithinPattern, patternVO.positionInParts - patternVO.length );
				}
				
				/** Update playback position in pattern (after all processing is done). */
				patternVO.positionInParts = ( patternVO.positionInParts + tempoVO.bufferLengthInPartsAsNumber ) % patternVO.length;
			}
		}

		/**
		 * 
		 */
		private function processPattern ( patternVO : PatternVO, firstPart : uint, lastPart : uint, playbackPositionInParts : Number ) : void
		{
			/** Create view update data object, used to update the pattern view. */
			var viewVO : PatternViewVO = new PatternViewVO( );
			viewVO.patternID = patternVO.id;
			viewVO.positionInParts = patternVO.positionInParts;
			viewVO.timeAtProcessStart = _playbackVO.timeAtProcessStart;
			
			/** Reset event counter when new loop starts. */
			if( firstPart == 0 )
			{
				patternVO.currentEventIndex = 0;
			}
			
			var trackVO : TrackVO = getTrackByID( patternVO.trackID );
			var numEvents : uint = patternVO.events.length;
			
			/** Loop through events in the pattern. */
			while( patternVO.currentEventIndex < numEvents )
			{
				var seqEventVO : SeqEventVO = patternVO.events[ patternVO.currentEventIndex ];
				
				if( seqEventVO.time <= lastPart )
				{
					
					/** The offset is the difference between the start of the buffer and the start time of the event. */
					var offsetInParts : Number = seqEventVO.time - playbackPositionInParts;
					var offsetInSamples : int = int( offsetInParts * _tempoVO.samplesPerPart );
					
					sendNotification( AudioNotifications.MESSAGE_TO_INSTRUMENT, new AudioEventVO( seqEventVO.message, trackVO.instrumentID, offsetInSamples ) );
					
					patternVO.currentEventIndex++;
					
					/**
					 * To animate a hit the following data is needed:
					 * seqEventVO.time / quantization = step index, to know which event triggered.
					 * offset * TempoVO.millisecondsPerPart, offset in milliseconds for correct timing.
					 */
					var eventViewVO : PatternEventViewVO = new PatternEventViewVO( );
					eventViewVO.stepIndex = seqEventVO.time / patternVO.quantization;
					eventViewVO.offsetInMs = offsetInParts * tempoVO.millisecondsPerPart;
					viewVO.events.push( eventViewVO );
				}
				else
				{
					break;
				}
			}
			
			/** Send view update notification to view. */
			sendNotification( SeqNotifications.SEQ_UPDATE_PATTERN_VIEW, viewVO );
		}
	}
}
