package com.hisschemoller.sequencer.view 
{
	import com.hisschemoller.audio.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.audio.notification.AudioNotifications;
	import com.hisschemoller.sequencer.model.vo.PlaybackVO;
	import com.hisschemoller.sequencer.model.vo.SeqEventVO;
	import com.hisschemoller.sequencer.model.vo.SequenceVO;
	import com.hisschemoller.sequencer.model.vo.TempoVO;
	import com.hisschemoller.sequencer.model.vo.TrackVO;
	import com.hisschemoller.sequencer.notification.SeqNotifications;
	import com.hisschemoller.sequencer.view.components.linearpattern.PatternEditor;
	import com.hisschemoller.sequencer.view.components.linearpattern.PatternEditorTrack;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.sequencer.view.events.SequencerEvent;

	import org.puremvc.as3.multicore.interfaces.INotification;
	import org.puremvc.as3.multicore.patterns.mediator.Mediator;

	import flash.events.Event;
	/**
	 * @author wouter.hisschemoller
	 * Oct 6, 2010
	 */
	public class PatternEditorMediator extends Mediator
	{
		public static const NAME : String = "PatternEditorMediator";
		
		public function PatternEditorMediator ( viewComponent : Object )
		{
			super( NAME, viewComponent );
		}

		/**
		 * 
		 */
		override public function listNotificationInterests () : Array
		{
			return [ AudioNotifications.ENTER_FRAME, 
					 SeqNotifications.SEQ_EVENTS_PROCESSED, 
					 SeqNotifications.SEQ_EVENT_CREATED, 
					 SeqNotifications.SEQ_EVENT_DELETED, 
					 SeqNotifications.SEQ_TRACK_ADDED, 
					 SeqNotifications.SEQ_SEQUENCE_SELECTED,
					 SeqNotifications.SEQ_TEMPO_CHANGED,
					 SeqNotifications.SEQ_PLAYBACK_CHANGED ];
		}

		/**
		 * 
		 */
		override public function handleNotification ( notification : INotification ) : void
		{
			switch(notification.getName( ))
			{
				case AudioNotifications.ENTER_FRAME:
					view.updateTime( (notification.getBody( ) as AudioStreamInfoVO).latencyInMS );
					break;
					
				case SeqNotifications.SEQ_EVENTS_PROCESSED:
					view.updatePlayback( notification.getBody( ) as PlaybackVO );
					break;
					
				case SeqNotifications.SEQ_EVENT_CREATED:
					view.addEvent( notification.getBody( ) as SeqEventVO );
					break;
					
				case SeqNotifications.SEQ_EVENT_DELETED:
					view.deleteEvent( notification.getBody( ) as SeqEventVO );
					break;
					
				case SeqNotifications.SEQ_TRACK_ADDED:
					view.addTrack( ( notification.getBody( ) as TrackVO ).id );
					break;
					
				case SeqNotifications.SEQ_SEQUENCE_SELECTED:
					view.changeSequence( notification.getBody( ) as SequenceVO );
					break;
					
				case SeqNotifications.SEQ_TEMPO_CHANGED:
					view.updateTempo( notification.getBody( ) as TempoVO );
					break;
					
				case SeqNotifications.SEQ_PLAYBACK_CHANGED:
					view.updatePlaybackState( notification.getBody( ) as String );
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister () : void
		{
			view.addEventListener( SequencerEvent.EVENT_CREATE, sequencerEventHandler );
			view.addEventListener( SequencerEvent.EVENT_DELETE, sequencerEventHandler );
			view.addEventListener( UIEventType.TRACK_EDIT_CLICK, editClickHandler );
		}

		/**
		 * 
		 */
		override public function onRemove () : void
		{
			view.removeEventListener( SequencerEvent.EVENT_CREATE, sequencerEventHandler );
			view.removeEventListener( SequencerEvent.EVENT_DELETE, sequencerEventHandler );
			view.removeEventListener( UIEventType.TRACK_EDIT_CLICK, editClickHandler );
		}

		/**
		 * 
		 */
		private function sequencerEventHandler (event : SequencerEvent) : void
		{
			switch(event.type)
			{
				case SequencerEvent.EVENT_CREATE:
					sendNotification( SeqNotifications.SEQ_CREATE_EVENT, event.seqEventVO );
					break;
				case SequencerEvent.EVENT_DELETE:
					sendNotification( SeqNotifications.SEQ_DELETE_EVENT, event.seqEventVO );
					break;
			}
		}

		/**
		 * Click to edit track settings.
		 */
		private function editClickHandler (event : Event) : void 
		{
			var trackID : String = (event.target as PatternEditorTrack).id;
			sendNotification( SeqNotifications.SEQ_EDIT_PATTERN_SETTINGS, trackID );
		}

		/** 
		 * 
		 */
		private function get view () : PatternEditor
		{
			return viewComponent as PatternEditor;
		}
	}
}
