package com.hisschemoller.sequencer.view.components.euclidpattern 
{
	import com.bit101.components.Label;
	import com.bit101.components.Panel;
	import com.hisschemoller.sequencer.model.vo.EuclidPatternVO;
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.model.vo.PatternViewVO;
	import com.hisschemoller.sequencer.model.vo.TempoVO;
	import com.hisschemoller.sequencer.view.enum.UIEventType;
	import com.hisschemoller.sequencer.view.enum.UILayout;
	import com.hisschemoller.utils.IDisposable;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;

	/**
	 * @author wouterhisschemoller
	 */
	public class EuclidEditor extends Panel implements IDisposable 
	{
		private var _patterns : Vector.<EuclidPattern> = new Vector.<EuclidPattern>( );
		private var _millisecondsPerPart : Number;
		
		public function EuclidEditor (parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0)
		{
			super( parent, xpos, ypos );
		}

		public function dispose () : void
		{
			_background.removeEventListener( MouseEvent.DOUBLE_CLICK, doubleClickHandler );
			
			var n : int = _patterns.length;
			while( --n > -1 )
			{
				var pattern : EuclidPattern = _patterns[ n ];
				pattern.removeEventListener( UIEventType.PATTERN_MOVE, patternMoveHandler );
			}
		}

		/**
		 * Update animations (on ENTER_FRAME).
		 */
		public function updateTime ( latencyInMs : Number ) : void
		{
			var n : int = _patterns.length;
			while( --n > -1 )
			{
				_patterns[ n ].updateView( latencyInMs );
			}
		}

		/**
		 * From SequencerProxy after a pattern is processed.
		 */
		public function updateViewData ( patternViewVO : PatternViewVO ) : void
		{
			var n : int = _patterns.length;
			while( --n > -1 )
			{
				if( _patterns[ n ].id == patternViewVO.patternID )
				{
					_patterns[ n ].updateViewData ( patternViewVO );
				}
			};
		}

		/**
		 * Tempo has changed. 
		 * _millisecondsPerPart is used in the time indicator animation.
		 */
		public function updateTempo (tempoVO : TempoVO) : void
		{
			_millisecondsPerPart = tempoVO.millisecondsPerPart;
			
			var n : int = _patterns.length;
			while( --n > -1 )
			{
				_patterns[ n ].updateTempo( _millisecondsPerPart );
			}
		}

		/**
		 * Create a new pattern.
		 */
		public function addPattern ( patternVO : EuclidPatternVO ) : void
		{
			var pattern : EuclidPattern = new EuclidPattern( this );
			pattern.createPattern( patternVO );
			pattern.updateTempo( _millisecondsPerPart );
			pattern.addEventListener( UIEventType.PATTERN_MOVE, patternMoveHandler );
			
			_patterns.push( pattern );
		}

		/**
		 * Update a pattern with changed data.
		 */
		public function updatePattern ( patternVO : EuclidPatternVO ) : void
		{
			var n : int = _patterns.length;
			while( --n > -1 )
			{
				if( patternVO.id == _patterns[ n ].id )
				{
					_patterns[ n ].updatePattern( patternVO );
					break;
				}
			}
		}

		/**
		 * Show selected indicator on pattern.
		 */
		public function selectPattern ( patternVO : PatternVO ) : void
		{
			var n : int = _patterns.length;
			while( --n > -1 )
			{
				if( patternVO.id == _patterns[ n ].id )
				{
					_patterns[ n ].select( true );
				}
				else
				{
					_patterns[ n ].select( false );
				}
			}
		}

		/**
		 * Component init, part of the startup cyclus.
		 */
		override protected function init () : void
		{
			super.init( );
			
			_background.doubleClickEnabled = true;
			_background.addEventListener( MouseEvent.DOUBLE_CLICK, doubleClickHandler );

			move( UILayout.MARGIN, UILayout.CONTROLS_HEIGHT + UILayout.MARGIN + UILayout.VERTICAL_GAP );
			setSize( stage.stageWidth - ( UILayout.MARGIN * 2 ), stage.stageHeight - UILayout.CONTROLS_HEIGHT - ( UILayout.MARGIN * 2 ) - UILayout.VERTICAL_GAP );
		}

		override protected function addChildren () : void
		{
			super.addChildren( );
			
			var label : Label = new Label( this, 20, 20, "Doubleclick this panel to create a new pattern.\n\nDoubleclick a pattern's center circle to edit its properties.\nDrag the center circle to move the pattern.");
			label.textField.multiline = true;
		}

		private function doubleClickHandler (event : MouseEvent) : void 
		{
			dispatchEvent( new Event( UIEventType.CREATE_PATTERN ) );
		}

		private function patternMoveHandler (event : Event) : void 
		{
			var pattern : EuclidPattern = event.target as EuclidPattern;
			pattern.x = Math.max( 0, Math.min( mouseX - pattern.mouseOffsetX, _background.width ) );
			pattern.y = Math.max( 0, Math.min( mouseY - pattern.mouseOffsetY, _background.height ) );
		}
	}
}
