package com.hisschemoller.sequencer.view.components.linearpattern 
{
	import com.hisschemoller.sequencer.model.vo.PatternVO;
	import com.hisschemoller.sequencer.model.vo.SeqEventVO;
	import com.hisschemoller.sequencer.view.events.SequencerEvent;
	import com.newgonzo.midi.messages.MessageStatus;

	import flash.display.DisplayObjectContainer;
	import flash.display.Shape;
	import flash.events.MouseEvent;

	/**
	 * @author wouterhisschemoller
	 */
	public class PatternEditorPatternFixed extends PatternEditorPatternBase
	{
		private var _dividers : Shape;
		
		public function PatternEditorPatternFixed (parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0, h : Number = 0)
		{
			super( parent, xpos, ypos, h );
		}

		/**
		 * Clean up before deletion.
		 */
		public function dispose () : void
		{
			_background.removeEventListener( MouseEvent.DOUBLE_CLICK, backgroundClickHandler );
		}
		
		/**
		 * @param patternVO: The pattern to display.
		 * @param zoomLevel: 1 means 1 pixel = 1 tick.
		 * (Quantization == 3 is 1 / 32 steps.)
		 */
		override public function updatePattern(patternVO : PatternVO) : void
		{
			super.updatePattern(patternVO);
			
			var numSteps : uint = _length / _quantization;
			var stepWidth : Number = (_length / numSteps) * _zoomLevel;
			
			/** Draw the sequencer steps. */
			var numEvents : uint = patternVO.events.length;
			for(var i : uint = 0; i < numEvents; i++ ) 
			{
				var seqEventVO : SeqEventVO = patternVO.events[i];

				/** NOTE_ON events are represented as selected steps. */
				if( seqEventVO.message.status == MessageStatus.NOTE_ON ) 
				{
					var xPos : Number = (seqEventVO.time / patternVO.quantization) * stepWidth;
					var step : PatternEditorStep = new PatternEditorStep( seqEventVO, this, xPos, 0 );
					step.addEventListener( MouseEvent.CLICK, stepClickHandler );
					_steps.push(step);
				}
			}
		}

		/**
		 * Zoom level changed.
		 */
		override public function updateZoomLevel (zoomLevel : Number) : void
		{
			super.updateZoomLevel( zoomLevel );
			
			/** Draw the quantization dividers. */
			_dividers.graphics.clear();
			_dividers.graphics.lineStyle(1, 0xCCCCCC);
			var numSteps : uint = _length / _quantization;
			var stepWidth : Number = (_length / numSteps) * _zoomLevel;
			for(var i : uint = 1; i < numSteps; i++ ) 
			{
				_dividers.graphics.moveTo(i * stepWidth, 0);
				_dividers.graphics.lineTo(i * stepWidth, _height);
			}
			
			/** Update the step positions. */
			var n : int = _steps.length;
			while( --n > -1 )
			{
				var step : PatternEditorStep = _steps[n];
				step.x = ( step.seqEventVO.time / _length ) * ( _length * _zoomLevel );
				step.setSize( stepWidth, _height );
			}
		}

		/**
		 * 
		 */
		override public function addStep(seqEventVO : SeqEventVO) : PatternEditorStep
		{
			var step : PatternEditorStep = super.addStep(seqEventVO);
			step.addEventListener( MouseEvent.CLICK, stepClickHandler );
			return step;
		}

		/**
		 * Component init, part of the startup cyclus.
		 */
		override protected function init () : void
		{
			super.init( );
			
			_background.addEventListener( MouseEvent.CLICK, backgroundClickHandler );
		}

		/**
		 * Add displayObjects, part of the startup cyclus.
		 */
		override protected function addChildren () : void 
		{
			super.addChildren( );
			
			_dividers = new Shape();
			addChild(_dividers);
		}
		
		/**
		 * 
		 */
		private function backgroundClickHandler (event : MouseEvent) : void 
		{
			createNewEvent(event.localX);
		}
		
		/**
		 * 
		 */
		private function stepClickHandler (event : MouseEvent) : void 
		{
			var step : PatternEditorStep = event.target as PatternEditorStep;
			dispatchEvent(new SequencerEvent(SequencerEvent.EVENT_DELETE , step.seqEventVO, true) );
		}
	}
}
